{% load perms %}

{% if perms.netbox_dns.view_record %}
{% if related_address_records %}
{% include 'inc/panel_table.html' with table=related_address_records heading='Related DNS Address Records' %}
{% endif %}
{% if related_pointer_records %}
{% include 'inc/panel_table.html' with table=related_pointer_records heading='Related DNS Pointer Records' %}
{% endif %}
{% endif %}
